@@ -6,11 +6,11 @@ WWW::Mechanize - Handy web browsing in a Perl object
 
 =head1 VERSION
 
-Version 1.16
+Version 1.22
 
 =cut
 
-our $VERSION = "1.16";
+our $VERSION = '1.22';
 
 =head1 SYNOPSIS
 
@@ -61,7 +61,6 @@ If you want finer control over over your page fetching, you can use
 these methods. C<follow_link> and C<submit_form> are just high
 level wrappers around them.
 
-    $mech->follow( $link );
     $mech->find_link( n => $number );
     $mech->form_number( $number );
     $mech->form_name( $name );
@@ -86,6 +85,10 @@ FAQ in WWW::Mechanize::FAQ for more.
 
 The CPAN documentation page for Mechanize.
 
+=item * L<http://search.cpan.org/dist/WWW-Mechanize/lib/WWW/Mechanize/FAQ.pod>
+
+Frequently asked questions.  Make sure you read here FIRST.
+
 =item * L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=WWW-Mechanize>
 
 The RT queue for bugs & enhancements in Mechanize.  Click the "Report bug"
@@ -102,11 +105,14 @@ use HTTP::Request 1.30;
 use LWP::UserAgent 2.003;
 use HTML::Form 1.00;
 use HTML::TokeParser;
-use URI::URL;
-use UNIVERSAL qw( isa );
 
 use base 'LWP::UserAgent';
 
+our $HAS_ZLIB;
+BEGIN {
+    $HAS_ZLIB = eval 'use Compress::Zlib (); 1;';
+}
+
 =head1 CONSTRUCTOR AND STARTUP
 
 =head2 new()
@@ -143,7 +149,7 @@ Checks each request made to see if it was successful.  This saves you
 the trouble of manually checking yourself.  Any errors found are errors,
 not warnings.  Default is off.
 
-=item * C<< onwarn => \&func() >>
+=item * C<< onwarn => \&func >>
 
 Reference to a C<warn>-compatible function, such as C<< L<Carp>::carp >>,
 that is called when a warning needs to be shown.
@@ -154,7 +160,7 @@ it's probably better to use the C<quiet> method to control that behavior.
 If this value is not passed, Mech uses C<Carp::carp> if L<Carp> is
 installed, or C<CORE::warn> if not.
 
-=item * C<< onerror => \&func() >>
+=item * C<< onerror => \&func >>
 
 Reference to a C<die>-compatible function, such as C<< L<Carp>::croak >>,
 that is called when there's a fatal error.
@@ -192,7 +198,7 @@ sub new {
         onwarn      => \&WWW::Mechanize::_warn,
         onerror     => \&WWW::Mechanize::_die,
         quiet       => 0,
-        stack_depth => 0,
+        stack_depth => 867_5309,    # Arbitrarily humungous stack
         headers     => {},
     );
 
@@ -295,10 +301,10 @@ sub known_agent_aliases {
 
 =head1 PAGE-FETCHING METHODS
 
-=head2 $mech->get( $url )
+=head2 $mech->get( $uri )
 
 Given a URL/URI, fetches it.  Returns an L<HTTP::Response> object.
-I<$url> can be a well-formed URL string, a L<URI> object, or a
+I<$uri> can be a well-formed URL string, a L<URI> object, or a
 L<WWW::Mechanize::Link> object.
 
 The results are stored internally in the agent object, but you don't
@@ -308,7 +314,7 @@ is deprecated and subject to change in the future.
 C<get()> is a well-behaved overloaded version of the method in
 L<LWP::UserAgent>.  This lets you do things like
 
-    $mech->get( $url, ":content_file"=>$tempfile );
+    $mech->get( $uri, ":content_file"=>$tempfile );
 
 and you can rest assured that the parms will get filtered down
 appropriately.
@@ -325,9 +331,43 @@ sub get {
             ? URI->new_abs( $uri, $self->base )
             : URI->new( $uri );
 
+    # It appears we are returning a super-class method,
+    # but it in turn calls the request() method here in Mechanize
     return $self->SUPER::get( $uri->as_string, @_ );
 }
 
+=head2 $mech->put( $uri, 'content' => $content )
+
+PUTs I<$content> to $uri.  Returns an L<HTTP::Response> object.
+I<$uri> can be a well-formed URI string, a L<URI> object, or a
+L<WWW::Mechanize::Link> object.
+
+=cut
+
+sub put {
+    my $self = shift;
+    my $uri = shift;
+
+    $uri = $uri->url if ref($uri) eq 'WWW::Mechanize::Link';
+
+    $uri = $self->base
+            ? URI->new_abs( $uri, $self->base )
+            : URI->new( $uri );
+
+    # It appears we are returning a super-class method,
+    # but it in turn calls the request() method here in Mechanize
+    return $self->_SUPER_put( $uri->as_string, @_ );
+}
+
+
+# Added until LWP::UserAgent has it.
+sub _SUPER_put {
+    require HTTP::Request::Common;
+    my($self, @parameters) = @_;
+    my @suff = $self->_process_colonic_headers(\@parameters,1);
+    return $self->request( HTTP::Request::Common::PUT( @parameters ), @suff );
+}
+
 =head2 $mech->reload()
 
 Acts like the reload button in a browser: repeats the current
@@ -343,7 +383,7 @@ sub reload {
 
     return unless my $req = $self->{req};
 
-    $self->_update_page( $req, $self->_make_request( $req, @_ ) );
+    return $self->_update_page( $req, $self->_make_request( $req, @_ ) );
 }
 
 =head2 $mech->back()
@@ -379,7 +419,8 @@ sub success {
 
 =head2 $mech->uri()
 
-Returns the current URI.
+Returns the current URI as a L<URI> object. This object stringifies
+to the URI itself.
 
 =head2 $mech->response() / $mech->res()
 
@@ -409,7 +450,7 @@ objects.
 =head2 $mech->current_form()
 
 Returns the current form as an L<HTML::Form> object.  I'd call this
-C<form()> except that C<L<form()>> already exists and sets the current_form.
+C<form()> except that C<L<form()>> used to exist to set the current_form.
 
 =head2 $mech->links()
 
@@ -424,14 +465,18 @@ HTTP headers.
 
 =cut
 
-sub uri {           my $self = shift; return $self->{uri}; }
+sub uri {
+    my $self = shift;
+    return $self->response->request->uri;
+}
+
 sub res {           my $self = shift; return $self->{res}; }
 sub response {      my $self = shift; return $self->{res}; }
 sub status {        my $self = shift; return $self->{status}; }
 sub ct {            my $self = shift; return $self->{ct}; }
 sub base {          my $self = shift; return $self->{base}; }
 sub current_form {  my $self = shift; return $self->{form}; }
-sub is_html {       my $self = shift; return defined $self->{ct} && ($self->{ct} eq "text/html"); }
+sub is_html {       my $self = shift; return defined $self->{ct} && ($self->{ct} eq 'text/html'); }
 
 =head2 $mech->title()
 
@@ -488,10 +533,14 @@ but will likely be backwards-compatible when it does.
 sub content {
     my $self = shift;
     my $content = $self->{content};
-    return $content unless $self->is_html;
 
-    while ( my ($cmd, $arg) = splice(@_, 0, 2) ) {
-        if ($cmd eq 'format') {
+    if ( $self->is_html ) {
+        my %parms = @_;
+        if ( exists $parms{base_href} ) {
+            my $arg = (delete $parms{base_href}) || $self->base;
+            $content=~s/<head>/<head>\n<base href="$arg">/;
+        }
+        if ( my $arg = delete $parms{format} ) {
             if ($arg eq 'text') {
                 require HTML::TreeBuilder;
                 my $tree = HTML::TreeBuilder->new();
@@ -499,19 +548,16 @@ sub content {
                 $tree->eof();
                 $tree->elementify(); # just for safety
                 $content = $tree->as_text();
+                $tree->delete;
             }
             else {
-                $self->die( qq{Unknown format parameter "$arg"} );
-            };
+                $self->die( qq{Unknown "format" parameter "$arg"} );
+            }
         }
-        elsif ($cmd eq 'base_href') {
-            $arg ||= $self->base;
-            $content=~s/<head>/<head>\n<base href="$arg">/;
-        }
-        else {
+        for my $cmd ( sort keys %parms ) {
             $self->die( qq{Unknown named argument "$cmd"} );
         }
-    }
+    } # is HTML
 
     return $content;
 }
@@ -566,27 +612,20 @@ Returns the result of the GET method (an HTTP::Response object) if
 a link was found. If the page has no links, or the specified link
 couldn't be found, returns undef.
 
-This method is meant to replace C<< $mech->follow() >> which should
-not be used in future development.
-
 =cut
 
 sub follow_link {
     my $self = shift;
     my %parms = ( n=>1, @_ );
 
-    if ( $parms{n} eq "all" ) {
+    if ( $parms{n} eq 'all' ) {
         delete $parms{n};
-        $self->warn( qq{follow_link(n=>"all") is not valid} );
+        $self->warn( q{follow_link(n=>"all") is not valid} );
     }
 
-    my $response;
     my $link = $self->find_link(%parms);
-    if ( $link ) {
-        $response = $self->get( $link->url );
-    }
-
-    return $response;
+    return $self->get( $link->url ) if $link;
+    return;
 }
 
 =head2 $mech->find_link()
@@ -649,7 +688,7 @@ more than one tag, as in:
     $mech->find_link( tag_regex => qr/^(a|frame)$/ );
 
 The tags and attributes looked at are defined below, at
-L<$mech->find_link() : link format>.
+L<< $mech->find_link() : link format >>.
 
 =back
 
@@ -688,7 +727,7 @@ sub find_link {
     my $self = shift;
     my %parms = ( n=>1, @_ );
 
-    my $wantall = ( $parms{n} eq "all" );
+    my $wantall = ( $parms{n} eq 'all' );
 
     $self->_clean_keys( \%parms, qr/^(n|(text|url|url_abs|name|tag)(_regex)?)$/ );
 
@@ -725,16 +764,16 @@ sub _match_any_link_parms {
     # No conditions, anything matches
     return 1 unless keys %$p;
 
-    return if defined $p->{url}           and !($link->url eq $p->{url} );
-    return if defined $p->{url_regex}     and !($link->url =~ $p->{url_regex} );
-    return if defined $p->{url_abs}       and !($link->url_abs eq $p->{url_abs} );
-    return if defined $p->{url_abs_regex} and !($link->url_abs =~ $p->{url_abs_regex} );
-    return if defined $p->{text}          and !(defined($link->text) and $link->text eq $p->{text} );
-    return if defined $p->{text_regex}    and !(defined($link->text) and $link->text =~ $p->{text_regex} );
-    return if defined $p->{name}          and !(defined($link->name) and $link->name eq $p->{name} );
-    return if defined $p->{name_regex}    and !(defined($link->name) and $link->name =~ $p->{name_regex} );
-    return if defined $p->{tag}           and !($link->tag and $link->tag eq $p->{tag} );
-    return if defined $p->{tag_regex}     and !($link->tag and $link->tag =~ $p->{tag_regex} );
+    return if defined $p->{url}           && !($link->url eq $p->{url} );
+    return if defined $p->{url_regex}     && !($link->url =~ $p->{url_regex} );
+    return if defined $p->{url_abs}       && !($link->url_abs eq $p->{url_abs} );
+    return if defined $p->{url_abs_regex} && !($link->url_abs =~ $p->{url_abs_regex} );
+    return if defined $p->{text}          && !(defined($link->text) && $link->text eq $p->{text} );
+    return if defined $p->{text_regex}    && !(defined($link->text) && $link->text =~ $p->{text_regex} );
+    return if defined $p->{name}          && !(defined($link->name) && $link->name eq $p->{name} );
+    return if defined $p->{name_regex}    && !(defined($link->name) && $link->name =~ $p->{name_regex} );
+    return if defined $p->{tag}           && !($link->tag && $link->tag eq $p->{tag} );
+    return if defined $p->{tag_regex}     && !($link->tag && $link->tag =~ $p->{tag_regex} );
 
     # Success: everything that was defined passed.
     return 1;
@@ -756,7 +795,7 @@ sub _clean_keys {
         }
 
         my $key_regex = ( $key =~ /_regex$/ );
-        my $val_regex = ( ref($val) eq "Regexp" );
+        my $val_regex = ( ref($val) eq 'Regexp' );
 
         if ( $key_regex ) {
             if ( !$val_regex ) {
@@ -887,7 +926,7 @@ sub find_image {
     my $self = shift;
     my %parms = ( n=>1, @_ );
 
-    my $wantall = ( $parms{n} eq "all" );
+    my $wantall = ( $parms{n} eq 'all' );
 
     $self->_clean_keys( \%parms, qr/^(n|(alt|url|url_abs|tag)(_regex)?)$/ );
 
@@ -924,14 +963,14 @@ sub _match_any_image_parms {
     # No conditions, anything matches
     return 1 unless keys %$p;
 
-    return if defined $p->{url}           and !($image->url eq $p->{url} );
-    return if defined $p->{url_regex}     and !($image->url =~ $p->{url_regex} );
-    return if defined $p->{url_abs}       and !($image->url_abs eq $p->{url_abs} );
-    return if defined $p->{url_abs_regex} and !($image->url_abs =~ $p->{url_abs_regex} );
-    return if defined $p->{alt}           and !(defined($image->alt) and $image->alt eq $p->{alt} );
-    return if defined $p->{alt_regex}     and !(defined($image->alt) and $image->alt =~ $p->{alt_regex} );
-    return if defined $p->{tag}           and !($image->tag and $image->tag eq $p->{tag} );
-    return if defined $p->{tag_regex}     and !($image->tag and $image->tag =~ $p->{tag_regex} );
+    return if defined $p->{url}           && !($image->url eq $p->{url} );
+    return if defined $p->{url_regex}     && !($image->url =~ $p->{url_regex} );
+    return if defined $p->{url_abs}       && !($image->url_abs eq $p->{url_abs} );
+    return if defined $p->{url_abs_regex} && !($image->url_abs =~ $p->{url_abs_regex} );
+    return if defined $p->{alt}           && !(defined($image->alt) && $image->alt eq $p->{alt} );
+    return if defined $p->{alt_regex}     && !(defined($image->alt) && $image->alt =~ $p->{alt_regex} );
+    return if defined $p->{tag}           && !($image->tag && $image->tag eq $p->{tag} );
+    return if defined $p->{tag_regex}     && !($image->tag && $image->tag =~ $p->{tag_regex} );
 
     # Success: everything that was defined passed.
     return 1;
@@ -960,7 +999,7 @@ sub find_all_images {
 
 =head2 $mech->forms
 
-Lists all the forms on the current page.  Each form is an HTML::Form
+Lists all the forms on the current page.  Each form is an L<HTML::Form>
 object.  In list context, returns a list of all forms.  In scalar
 context, returns an array reference of all forms.
 
@@ -977,21 +1016,28 @@ sub forms {
 
 Selects the I<number>th form on the page as the target for subsequent
 calls to C<L<field()>> and C<L<click()>>.  Also returns the form that was
-selected.  Emits a warning and returns undef if there is no such
-form.  Forms are indexed from 1, so the first form is number 1,
-not zero.
+selected.  
+
+If it is found, the form is returned as an L<HTML::Form> object and set internally  
+for later used with Mech's form methods such as C<L<field()>> and C<L<click()>>.
+
+Emits a warning and returns undef if no form is found. 
+
+The first form is number 1, not zero.
 
 =cut
 
 sub form_number {
     my ($self, $form) = @_;
+    # XXX Should we die if no $form is defined? Same question for form_name()
+
     if ($self->{forms}->[$form-1]) {
         $self->{form} = $self->{forms}->[$form-1];
         return $self->{form};
     }
     else {
         $self->warn( "There is no form numbered $form" );
-        return;
+        return undef;
     }
 }
 
@@ -999,8 +1045,12 @@ sub form_number {
 
 Selects a form by name.  If there is more than one form on the page
 with that name, then the first one is used, and a warning is
-generated.  Also returns the form itself, or undef if it's not
-found.
+generated.  
+
+If it is found, the form is returned as an L<HTML::Form> object and set internally  
+for later used with Mech's form methods such as C<L<field()>> and C<L<click()>>.
+
+Returns undef if no form is found. 
 
 Note that this functionality requires libwww-perl 5.69 or higher.
 
@@ -1011,9 +1061,9 @@ sub form_name {
 
     my $temp;
     my @matches = grep {defined($temp = $_->attr('name')) and ($temp eq $form) } $self->forms;
-    if ( @matches ) {
-        $self->warn( "There are ", scalar @matches, " forms named $form.  The first one was used." )
-            if @matches > 1;
+    if ( my $nmatches = @matches ) {
+        $self->warn( "There are $nmatches forms named $form.  The first one was used." )
+            if $nmatches > 1;
         return $self->{form} = $matches[0];
     }
     else {
@@ -1022,13 +1072,52 @@ sub form_name {
     }
 }
 
+=head2 $mech->form_with_fields(@fields)
+
+Selects a form by passing in a list of field names it must contain.  If there
+is more than one form on the page with that matches, then the first one is used,
+and a warning is generated.
+
+If it is found, the form is returned as an L<HTML::Form> object and set internally
+for later used with Mech's form methods such as C<L<field()>> and C<L<click()>>.
+
+Returns undef if no form is found.
+
+Note that this functionality requires libwww-perl 5.69 or higher.
+
+=cut
+
+sub form_with_fields {
+    my ($self, @fields) = @_;
+    die 'no fields provided' unless scalar @fields;
+
+    my @matches;
+    FORMS: for my $form (@{ $self->forms }) {
+        my @fields_in_form = $form->param();
+        for my $field (@fields) {
+            next FORMS unless grep { $_ eq $field } @fields_in_form;
+        }
+        push @matches, $form;
+    }
+
+    if ( my $nmatches = @matches ) {
+        $self->warn( "There are $nmatches forms with the named fields.  The first one was used." )
+            if $nmatches > 1;
+        return $self->{form} = $matches[0];
+    }
+    else {
+        $self->warn( qq{There is no form with the requested fields} );
+        return undef;
+    }
+}
+
 
 =head2 $mech->field( $name, $value, $number )
 
 =head2 $mech->field( $name, \@values, $number )
 
 Given the name of a field, set its value to the value specified.  This
-applies to the current form (as set by the C<L<form()>> method or defaulting
+applies to the current form (as set by the L<form_name()> or L<form_number()> method or defaulting
 to the first form on the page).
 
 The optional I<$number> parameter is used to distinguish between two fields
@@ -1045,7 +1134,7 @@ sub field {
         $form->find_input($name, undef, $number)->value($value);
     }
     else {
-        if ( ref($value) eq "ARRAY" ) {
+        if ( ref($value) eq 'ARRAY' ) {
             $form->param($name, $value);
         }
         else {
@@ -1064,11 +1153,10 @@ C<$value> is an array, only the B<first> value will be set.  [Note:
 the documentation previously claimed that only the last value would
 be set, but this was incorrect.]  Passing C<$value> as a hash with
 an C<n> key selects an item by number (e.g. C<{n => 3> or C<{n => [2,4]}>).
-The numbering starts at 1.  This applies to the current form (as set by the
-C<L<form()>> method or defaulting to the first form on the page).
+The numbering starts at 1.  This applies to the current form.
 
 Returns 1 on successfully setting the value. On failure, returns
-undef and calls C<$self->warn()> with an error message.
+undef and calls C<< $self>warn() >> with an error message.
 
 =cut
 
@@ -1090,7 +1178,7 @@ sub select {
 
     # For $mech->select($name, {n => 3}) or $mech->select($name, {n => [2,4]}),
     # transform the 'n' number(s) into value(s) and put it in $value.
-    if (ref($value) eq "HASH") {
+    if (ref($value) eq 'HASH') {
         for (keys %$value) {
             $self->warn(qq{Unknown select value parameter "$_"})
               unless $_ eq 'n';
@@ -1134,9 +1222,9 @@ sub select {
             $self->warn('Hash value is invalid');
             return;
         }
-    }
+    } # hashref
 
-    if (ref($value) eq "ARRAY") {
+    if (ref($value) eq 'ARRAY') {
         $form->param($name, $value);
         return 1;
     }
@@ -1158,8 +1246,7 @@ which has the field value and its number as the 2 elements.
 
 The fields are numbered from 1.
 
-This applies to the current form (as set by the C<L<form()>> method or
-defaulting to the first form on the page).
+This applies to the current form.
 
 =cut
 
@@ -1167,7 +1254,7 @@ sub set_fields {
     my $self = shift;
     my %fields = @_;
 
-    my $form = $self->current_form or $self->die( "No form defined" );
+    my $form = $self->current_form or $self->die( 'No form defined' );
 
     while ( my ( $field, $value ) = each %fields ) {
         if ( ref $value eq 'ARRAY' ) {
@@ -1215,6 +1302,8 @@ C<OPTION> menu field to "Checking".
 The possible field specifier types are: "text", "password", "hidden",
 "textarea", "file", "image", "submit", "radio", "checkbox" and "option".
 
+C<set_visible> returns the number of values set. 
+
 =cut
 
 sub set_visible {
@@ -1225,6 +1314,7 @@ sub set_visible {
 
     my $num_set = 0;
     for my $value ( @_ ) {
+        # Handle type/value pairs an arrayref
         if ( ref $value eq 'ARRAY' ) {
             my ( $type, $value ) = @$value;
             while ( my $input = shift @inputs ) {
@@ -1236,6 +1326,7 @@ sub set_visible {
                 }
             } # while
         }
+        # by default, it's a value
         else {
             while ( my $input = shift @inputs ) {
                 next if $input->type eq 'hidden';
@@ -1251,7 +1342,7 @@ sub set_visible {
 
 =head2 $mech->tick( $name, $value [, $set] )
 
-'Ticks' the first checkbox that has both the name and value assoicated
+'Ticks' the first checkbox that has both the name and value associated
 with it on the current form.  Dies if there is no named check box for
 that value.  Passing in a false value as the third optional argument
 will cause the checkbox to be unticked.
@@ -1266,7 +1357,7 @@ sub tick {
 
     # loop though all the inputs
     my $index = 0;
-    while ( my $input = $self->current_form->find_input( $name, "checkbox", $index ) ) {
+    while ( my $input = $self->current_form->find_input( $name, 'checkbox', $index ) ) {
         # Can't guarantee that the first element will be undef and the second
         # element will be the right name
         foreach my $val ($input->possible_values()) {
@@ -1299,8 +1390,7 @@ sub untick {
 =head2 $mech->value( $name, $number )
 
 Given the name of a field, return its value. This applies to the current
-form (as set by the C<form()> method or defaulting to the first form on
-the page).
+form.
 
 The option I<$number> parameter is used to distinguish between two fields
 with the same name.  The fields are numbered from 1.
@@ -1396,7 +1486,10 @@ sub click_button {
         }
     }
 
-    for ($args{x}, $args{y}) { $_ = 1 unless defined; }
+    for ($args{x}, $args{y}) {
+        $_ = 1 unless defined;
+    }
+
     my $form = $self->{form};
     my $request;
     if ( $args{name} ) {
@@ -1449,6 +1542,16 @@ are a list of key/value pairs, all of which are optional.
 
 =over 4
 
+=item * fields => \%fields
+
+Probably all you need for the common case. It combines a smart form selector
+and data setting in one operation. It selects the first form that contains all
+fields mentioned in C<\%fields>.  This is nice because you don't need to know
+the name or number of the form to do this. 
+(calls C<L<form_with_fields>> and C<L<set_fields()>>).
+
+If you choose this, the form_number, form_name and fields options will be ignored.
+
 =item * form_number => n
 
 Selects the I<n>th form (calls C<L<form_number()>>).  If this parm is not
@@ -1458,9 +1561,9 @@ specified, the currently-selected form is used.
 
 Selects the form named I<name> (calls C<L<form_name()>>)
 
-=item * fields => fields
+=item * fields => \%fields
 
-Sets the field values from the I<fields> hashref (calls C<L<set_fields()>>)
+Sets the field values from the I<fields> hashref (calls C<L<set_fields()>>).
 
 =item * button => button
 
@@ -1484,24 +1587,42 @@ sub submit_form {
     my( $self, %args ) = @_ ;
 
     for ( keys %args ) {
-        if ( !/^(form_(number|name)|fields|button|x|y)$/ ) {
+        if ( !/^(form_(number|name|fields)|(with_)?fields|button|x|y)$/ ) {
+            # XXX Why not die here?  
             $self->warn( qq{Unknown submit_form parameter "$_"} );
         }
     }
 
-    if ( my $form_number = $args{'form_number'} ) {
+    my $fields;
+    for (qw/with_fields fields/) {
+        if ($args{$_}) {
+            if ( ref $args{$_} eq 'HASH' ) {
+                $fields = $args{$_};
+            }
+            else {
+                die "$_ arg to submit_form must be a hashref";
+            }
+            last;
+        }
+    }
+
+    if ($args{'with_fields'}) {
+        $fields || die q{must submit some 'fields' with with_fields};
+        $self->form_with_fields(keys %{$fields}) or die;
+    }
+    elsif ( my $form_number = $args{'form_number'} ) {
         $self->form_number( $form_number ) or die;
     }
     elsif ( my $form_name = $args{'form_name'} ) {
         $self->form_name( $form_name ) or die;
     }
-
-    if ( my $fields = $args{'fields'} ) {
-        if ( isa( $fields, 'HASH' ) ) {
-            $self->set_fields( %{$fields} ) ;
-        } # TODO: What if it's not a hash?  We just ignore it silently?
+    else {
+        # No form selector was used. 
+        # Maybe a form was set separately, or we'll default to the first form.
     }
 
+    $self->set_fields( %{$fields} ) if $fields;
+
     my $response;
     if ( $args{button} ) {
         $response = $self->click( $args{button}, $args{x} || 0, $args{y} || 0 );
@@ -1596,11 +1717,13 @@ sub quiet {
     return $self->{quiet};
 }
 
-=head2 $mech->stack_depth($value)
+=head2 $mech->stack_depth( $max_depth )
 
-Get or set the page stack depth. Older pages are discarded first.
+Get or set the page stack depth. Use this if you're doing a lot of page
+scraping and running out of memory.
 
-A value of 0 means "keep all the pages".
+A value of 0 means "no history at all."  By default, the max stack depth
+is humongously large, effectively keeping all history.
 
 =cut
 
@@ -1621,13 +1744,28 @@ sub save_content {
     my $self = shift;
     my $filename = shift;
 
-    open( my $fh, ">", $filename ) or $self->die( "Unable to create $filename: $!" );
-    print $fh $self->content;
+    open( my $fh, '>', $filename ) or $self->die( "Unable to create $filename: $!" );
+    print {$fh} $self->content;
     close $fh;
 }
 
 =head1 OVERRIDDEN LWP::UserAgent METHODS
 
+=head2 $mech->clone()
+
+Clone the mech object. We override here to be sure 
+the cookie jar gets copied over
+
+=cut 
+
+sub clone {
+    my $self = shift;
+    my $clone =  $self->SUPER::clone();
+    $clone->{cookie_jar} = $self->cookie_jar;
+    return $clone;
+}
+
+
 =head2 $mech->redirect_ok()
 
 An overloaded version of C<redirect_ok()> in L<LWP::UserAgent>.
@@ -1668,7 +1806,7 @@ sub request {
 
     $request = $self->_modify_request( $request );
 
-    if ( $request->method eq "GET" || $request->method eq "POST" ) {
+    if ( $request->method eq 'GET' || $request->method eq 'POST' ) {
         $self->_push_page_stack();
     }
 
@@ -1735,74 +1873,38 @@ sub update_html {
     return;
 }
 
-=head1 DEPRECATED METHODS
+=head2 $mech->credentials($username, $password)
 
-This methods have been replaced by more flexible and precise methods.
-Please use them instead.
+Provide credentials to be used for HTTP Basic authentication for all sites and
+realms until further notice.  
 
-=head2 $mech->follow($string|$num)
-
-B<DEPRECATED> in favor of C<L<follow_link()>>, which provides more
-flexibility.
-
-Follow a link.  If you provide a string, the first link whose text
-matches that string will be followed.  If you provide a number, it
-will be the I<$num>th link on the page.  Note that the links are
-0-based.
-
-Returns true if the link was found on the page or undef otherwise.
+The four argument form described in L<LWP::UserAgent> is still supported.
 
 =cut
 
-sub follow {
-    my ($self, $link) = @_;
-    my @links = $self->links;
-    my $thislink;
-    if ( $link =~ /^\d+$/ ) { # is a number?
-        if ($link <= $#links) {
-            $thislink = $links[$link];
-        }
-        else {
-            $self->warn( "Link number $link is greater than maximum link $#links on this page ($self->{uri})" );
-            return;
+{
+    my $saved_method;
+
+    sub credentials {
+        my $self = shift;
+        no warnings 'redefine'; ## no critic
+
+        if (@_ == 4) {
+            $saved_method
+                and *LWP::UserAgent::get_basic_credentials = $saved_method;
+            return $self->SUPER::credentials(@_);
         }
+
+        @_ == 2
+            or $self->die( 'Invalid # of args for overridden credentials()' );
+
+        my ($username, $password) = @_;
+        $saved_method ||= \&LWP::UserAgent::get_basic_credentials;
+        *LWP::UserAgent::get_basic_credentials
+            = sub { return $username, $password };
     }
-    else {                        # user provided a regexp
-        LINK: foreach my $l (@links) {
-            if ( defined($l->[1]) && $l->[1] =~ /$link/) {
-                $thislink = $l;     # grab first match
-                last LINK;
-            }
-        }
-        unless ($thislink) {
-            $self->warn( "Can't find any link matching $link on this page ($self->{uri})" );
-            return;
-        }
-    }
-
-    $thislink = $thislink->[0];     # we just want the URL, not the text
-
-    $self->get( $thislink );
-
-    return 1;
 }
 
-=head2 $mech->form($number|$name)
-
-B<DEPRECATED> in favor of C<L<form_name()>> or C<L<form_number()>>.
-
-Selects a form by number or name, depending on if it gets passed an
-all-numeric string or not.  This means that if you have a form name
-that's all digits, this method will not do the right thing.
-
-=cut
-
-sub form {
-    my $self = shift;
-    my $arg = shift;
-
-    return $arg =~ /^\d+$/ ? $self->form_number($arg) : $self->form_name($arg);
-}
 
 =head1 INTERNAL-ONLY METHODS
 
@@ -1825,28 +1927,33 @@ sub _update_page {
 
     $self->{res} = $res;
 
-    # These internal hash elements should be dropped in favor of
-    # the accessors soon. -- 1/19/03
     $self->{status}  = $res->code;
     $self->{base}    = $res->base;
-    $self->{ct}      = $res->content_type || "";
+    $self->{ct}      = $res->content_type || '';
 
     if ( $res->is_success ) {
         $self->{uri} = $self->{redirected_uri};
         $self->{last_uri} = $self->{uri};
     }
-    else {
+
+    if ( $res->is_error ) {
         if ( $self->{autocheck} ) {
-            $self->die( "Error ", $request->method, "ing ", $request->uri, ": ", $res->message );
+            $self->die( 'Error ', $request->method, 'ing ', $request->uri, ': ', $res->message );
         }
     }
 
     $self->_reset_page;
+
+    # Try to decode the content. Undef will be returned if there's nothing to decompress.
+    # See docs in HTTP::Message for details. Do we need to expose the options there? 
+    my $content = $res->decoded_content;
+       $content = $res->content if (not defined $content);
+
     if ($self->is_html) {
-        $self->update_html($res->content);
+        $self->update_html($content);
     }
     else {
-        $self->{content} = $res->content;
+        $self->{content} = $content;
     }
 
     return $res;
@@ -1855,7 +1962,11 @@ sub _update_page {
 
 =head2 $mech->_modify_request( $req )
 
-Modifies the request according to all the internal header mangling.
+Modifies a L<HTTP::Request> before the request is sent out,
+for both GET and POST requests.
+
+We add a C<Referer> header, as well as header to note that we can accept gzip
+encoded content, if L<Compress::Zlib> is installed.
 
 =cut
 
@@ -1865,9 +1976,10 @@ sub _modify_request {
 
     # add correct Accept-Encoding header to restore compliance with
     # http://www.freesoft.org/CIE/RFC/2068/158.htm
-    unless ( $req->header( 'Accept-Encoding' ) ) {
-        # Only allow "identity" for the time being
-        $req->header( 'Accept-Encoding', 'identity' );
+    # http://use.perl.org/~rhesa/journal/25952
+    if (not $req->header( 'Accept-Encoding' ) ) {
+        # "identity" means "please! unencoded content only!"
+        $req->header( 'Accept-Encoding', $HAS_ZLIB ? 'gzip' : 'identity' );
     }
 
     my $last = $self->{last_uri};
@@ -1927,11 +2039,11 @@ property with L<WWW::Mechanize::Link> objects.
 =cut
 
 my %link_tags = (
-    a => "href",
-    area => "href",
-    frame => "src",
-    iframe => "src",
-    meta => "content",
+    a => 'href',
+    area => 'href',
+    frame => 'src',
+    iframe => 'src',
+    meta => 'content',
 );
 
 sub _extract_links {
@@ -1954,8 +2066,8 @@ sub _extract_links {
 
 
 my %image_tags = (
-    img => "src",
-    input => "src",
+    img => 'src',
+    input => 'src',
 );
 
 sub _extract_images {
@@ -1984,9 +2096,9 @@ sub _image_from_token {
     my $tag = $token->[0];
     my $attrs = $token->[1];
 
-    if ( $tag eq "input" ) {
+    if ( $tag eq 'input' ) {
         my $type = $attrs->{type} or return;
-        return unless $type eq "image";
+        return unless $type eq 'image';
     }
 
     require WWW::Mechanize::Image;
@@ -2013,9 +2125,9 @@ sub _link_from_token {
 
     my $text;
     my $name;
-    if ( $tag eq "a" ) {
+    if ( $tag eq 'a' ) {
         $text = $parser->get_trimmed_text("/$tag");
-        $text = "" unless defined $text;
+        $text = '' unless defined $text;
 
         my $onClick = $attrs->{onclick};
         if ( $onClick && ($onClick =~ /^window\.open\(\s*'([^']+)'/) ) {
@@ -2029,10 +2141,10 @@ sub _link_from_token {
 
     $name = $attrs->{name};
 
-    if ( $tag eq "meta" ) {
-        my $equiv = $attrs->{"http-equiv"};
-        my $content = $attrs->{"content"};
-        return unless $equiv && (lc $equiv eq "refresh") && defined $content;
+    if ( $tag eq 'meta' ) {
+        my $equiv = $attrs->{'http-equiv'};
+        my $content = $attrs->{'content'};
+        return unless $equiv && (lc $equiv eq 'refresh') && defined $content;
 
         if ( $content =~ /^\d+\s*;\s*url\s*=\s*(\S+)/i ) {
             $url = $1;
@@ -2073,20 +2185,16 @@ sub _push_page_stack {
     my $self = shift;
 
     # Don't push anything if it's a virgin object
-    if ( $self->{res} ) {
+    if ( $self->{res} && $self->stack_depth ) {
         my $save_stack = $self->{page_stack};
         $self->{page_stack} = [];
 
         my $clone = $self->clone;
-        # Huh, LWP::UserAgent->clone() ditches cookie_jar? Copy it over now.
-        $clone->{cookie_jar} = $self->cookie_jar;
-        push( @$save_stack, $clone );
+        push( @{$save_stack}, $clone );
 
-        if ( $self->stack_depth > 0 ) {
-            while ( @$save_stack > $self->stack_depth ) {
-                shift @$save_stack;
-            }
-        } # if stack_depth > 0
+        while ( @{$save_stack} > $self->stack_depth ) {
+            shift @{$save_stack};
+        }
         $self->{page_stack} = $save_stack;
     }
 
@@ -2096,16 +2204,16 @@ sub _push_page_stack {
 sub _pop_page_stack {
     my $self = shift;
 
-    if (@{$self->{page_stack}}) {
+    if ( $self->{page_stack} && @{$self->{page_stack}} ) {
         my $popped = pop @{$self->{page_stack}};
 
         # eliminate everything in self
-        foreach my $key ( keys %$self ) {
+        foreach my $key ( keys %{$self} ) {
             delete $self->{ $key }              unless $key eq 'page_stack';
         }
 
         # make self just like the popped object
-        foreach my $key ( keys %$popped ) {
+        foreach my $key ( keys %{$popped} ) {
             $self->{ $key } = $popped->{ $key } unless $key eq 'page_stack';
         }
     }
@@ -2117,7 +2225,7 @@ sub _pop_page_stack {
 
 Centralized warning method, for diagnostics and non-fatal problems.
 Defaults to calling C<CORE::warn>, but may be overridden by setting
-C<onwarn> in the construcotr.
+C<onwarn> in the constructor.
 
 =cut
 
@@ -2128,7 +2236,7 @@ sub warn {
 
     return if $self->quiet;
 
-    $handler->(@_);
+    return $handler->(@_);
 }
 
 =head2 die( @messages )
@@ -2143,22 +2251,24 @@ sub die {
 
     return unless my $handler = $self->{onerror};
 
-    $handler->(@_);
+    return $handler->(@_);
 }
 
 
 # NOT an object method!
 sub _warn {
     require Carp;
-    &Carp::carp; # pass thru
+    &Carp::carp; ## no critic
 }
 
 # NOT an object method!
 sub _die {
     require Carp;
-    &Carp::croak; # pass thru
+    &Carp::croak; ## no critic
 }
 
+1; # End of module
+
 __END__
 
 =head1 WWW::MECHANIZE'S SUBVERSION REPOSITORY
@@ -2200,11 +2310,6 @@ L<http://books.slashdot.org/article.pl?sid=03/12/11/2126256>
 
 =over 4
 
-=item * WWW::Mechanize Development mailing list
-
-Hosted at Sourceforge, this is where the contributors to Mech
-discuss things.  L<http://sourceforge.net/mail/?group_id=83309>
-
 =item * LWP mailing list
 
 The LWP mailing list is at
@@ -2213,6 +2318,11 @@ user-oriented and well-populated than the WWW::Mechanize Development
 list.  This is a good list for Mech users, since LWP is the basis
 for Mech.
 
+=item * WWW::Mechanize Development mailing list
+
+Hosted at Sourceforge, this is where the contributors to Mech
+discuss things.  L<http://sourceforge.net/mail/?group_id=83309>
+
 =item * L<WWW::Mechanize::Examples>
 
 A random array of examples submitted by users, included with the
@@ -2224,6 +2334,10 @@ Mechanize distribution.
 
 =over 4
 
+=item * L<http://www-128.ibm.com/developerworks/linux/library/wa-perlsecure.html>
+
+IBM article "Secure Web site access with Perl"
+
 =item * L<http://www.oreilly.com/catalog/googlehks2/chapter/hack84.pdf>
 
 Leland Johnson's hack #84 in I<Google Hacks, 2nd Edition> is
@@ -2350,14 +2464,13 @@ Joshua Gatcomb,
 Julien Beasley,
 Abe Timmerman,
 Peter Stevens,
+Pete Krawczyk,
 and the late great Iain Truskett.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005 Andy Lester. All rights reserved. This program is
+Copyright (c) 2006 Andy Lester. All rights reserved. This program is
 free software; you can redistribute it and/or modify it under the same
 terms as Perl itself.
 
 =cut
-
-1;
